import { nextPowerOf2 } from '../../util';
import { Renderer } from '../renderer/renderer';
import { SpriteCoords } from '../sprite';

export function loadImage(url: string): Promise<HTMLImageElement> {
	return new Promise((resolve, reject) => {
		const img = new Image();
		img.addEventListener('load', () => { resolve(img); });
		img.addEventListener('error', () => { reject(new Error('failed to load ' + url)); });
		img.src = url;
	});
}

export function mkTexture(
	r: Renderer,
	src: HTMLCanvasElement|HTMLImageElement,
	frame: number = 0,
	total: number = 1,
): [WebGLTexture, [number, number], SpriteCoords] {
	const gl = r.getContext();
	const texture = gl.createTexture();
	if (!texture)
		throw new Error('unable to create texture');

	const w = src.width, h = src.height / total;
	const w2 = nextPowerOf2(w), h2 = nextPowerOf2(h);

	const canvas = document.createElement('canvas');
	canvas.width = w2;
	canvas.height = h2;

	const ctx = canvas.getContext('2d') as CanvasRenderingContext2D;
	ctx.drawImage(src, 0, frame * h, w, h, 0, 0, w, h);

	gl.bindTexture(gl.TEXTURE_2D, texture);
	gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, canvas);
	gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
	gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
	gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.NEAREST);
	gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.NEAREST);

	const size: [number, number] = [
		w / r.coordScale, h / r.coordScale,
	];
	const coords: SpriteCoords = [
		0, 0, w2 / r.coordScale, h2 / r.coordScale,
	];

	return [texture, size, coords];
}
